%This fuction computes the optimal threshold of the 2LPS and the
%corresponding expected response time
%Parameters:
%A, B: description of the phase type distribution
%body: number of phases for the body
%tail: number of phases for the tail
%thmin, thmax: range of search of the optimal thereshold
%rho: load factor of the queue, 0<rho<1
%tol: tolerance

function [th,resp] = optimalThreshold(A, B, body, tail, thmin, thmax, rho, tol)
    

    %Create the generalised hyper exponential distribution
    [p,u] = convertPhfit(A, B, body, tail);
    
    %Compute the mean of the distribution (notitce mu is not a rate)
    mu = 0;

    for i=1:length(p)
        mu = mu + p(i) / u(i); 
    end
    
    %Compute the arrival rate
    l = rho * (1 / mu);
    
    %compute the optimatal thereshold
    %by a golden section search
    
    a = thmin;
    b = thmax;
    h = b - a;
    
    if (h<=tol)
        th = (a+b)/2;
        resp = averageR(u, p, l, th);
    else
        invphi = (sqrt(5) - 1) / 2;
        invphi2 = (3 - sqrt(5)) / 2;
        n = ceil(log(tol/h)/log(invphi));
        
        c = a + invphi2 * h;
        d = a + invphi * h;
        yc = averageR(u, p, l, c);
        yd = averageR(u, p, l, d);
        
        for k = 0:n-1
            if (yc < yd)
                b = d;
                d = c;
                yd = yc;
                h = invphi * h;
                c = a + invphi2 * h;
                yc = averageR(u, p, l, c);
            else
                a = c;
                c = d;
                yc = yd;
                h = invphi * h;
                d = a + invphi * h;
                yd = averageR(u, p, l, d);
            end
        end
        
        if (yc<yd)
            th = (a+d)/2;
        else
            th = (c+b)/2;
        end
        
        resp = averageR(u, p, l, th);
    end
end

